package org.fuseri.modulelanguageschool.course;

import org.fuseri.model.dto.course.CourseCreateDto;
import org.fuseri.model.dto.course.CourseDto;
import org.fuseri.model.dto.course.LanguageTypeDto;
import org.fuseri.model.dto.course.ProficiencyLevelDto;
import org.fuseri.model.dto.user.UserDto;
import org.fuseri.modulelanguageschool.user.UserMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

@Service
@Transactional
public class CourseFacade {

    @Autowired
    private CourseService courseService;

    @Autowired
    private CourseMapper courseMapper;

    @Autowired
    private UserMapper userMapper;

//    @Autowired
//    public CourseFacade(CourseService courseService, CourseMapper courseMapper, UserMapper userMapper) {
//        this.courseService = courseService;
//        this.courseMapper = courseMapper;
//        this.userMapper = userMapper;
//    }

    @Transactional
    public CourseDto create(CourseCreateDto dto) {
        return courseMapper.mapToDto(courseService.save(courseMapper.mapToCourse(dto)));
    }

    @Cacheable(cacheNames = "courses", key = "#id")
    @Transactional(readOnly = true)
    public CourseDto findById(Long id) {
        return courseMapper.mapToDto(courseService.findById(id));
    }

    @Transactional(readOnly = true)
    public Page<CourseDto> findAll(Pageable pageable) {
        return courseMapper.mapToPageDto(courseService.findAll(pageable));
    }

    @Transactional
    public CourseDto update(Long id, CourseCreateDto dto) {
        return courseMapper.mapToDto(courseService.update(id, courseMapper.mapToCourse(dto)));
    }

    @Transactional
    public void delete(Long id) {
        courseService.delete(id);
    }

    public List<CourseDto> findAll(LanguageTypeDto lang) {
        return courseMapper.mapToList(courseService.findAll(Language.valueOf(lang.name())));
    }

    public List<CourseDto> findAll(LanguageTypeDto lang, ProficiencyLevelDto prof) {
        return courseMapper.mapToList(courseService.findAll(Language.valueOf(lang.name()), ProficiencyLevel.valueOf(prof.name())));
    }

    public CourseDto enrol(Long id, UserDto student) {
        return courseMapper.mapToDto(courseService.enrol(id, userMapper.fromDto(student)));
    }

    public CourseDto expel(Long id, UserDto student) {
        return courseMapper.mapToDto(courseService.expel(id, userMapper.fromDto(student)));
    }
}
